(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uindex_hashtbl.ml,v 1.5 2014-01-28 12:19:24 fclement Exp $*)

(* Index hash tables. *)

module Index = Uindex_index;;
module Gen_hashtbl = Ugen_numbering_hashtbl;;

type size = Upervasives.length;;
type 'a key = 'a Index.element;;
type 'a value = 'a Index.index;;

type 'a t = (size, 'a key) Gen_hashtbl.t;;

external of_t : 'a t -> (size, 'a key) Gen_hashtbl.t = "%identity";;

(* Construction function. *)

let make a =
  Uarray.check_nonempty "Uindex_hashtbl.make" a;
  Uarray.check_strictly_increasing "Uindex_hashtbl.make" a;
  let len = Uarray.length a in
  let h k = Index.of_natural (Uarray.sorted_array_find a k) in
  let h_inv v = Array.get a (Index.to_int v) in
  { Gen_hashtbl.length = len; size = len; h = h; h_inv = h_inv; }
;;

(* Access functions. *)

let length = Gen_hashtbl.length;;

let mem ih k =
  try ignore (ih.Gen_hashtbl.h k); true with
  | Not_found -> false
;;

let value_of_key = Gen_hashtbl.value_of_key;;

let key_of_value = Gen_hashtbl.key_of_value;;

let equal = Gen_hashtbl.equal;;

(* Iterators. *)

let iter f = Gen_hashtbl.iter f;;

let sub_iter f = Gen_hashtbl.sub_iter f;;

let fold_left f = Gen_hashtbl.fold_left f;;
