(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uindex_universe.ml,v 1.6 2014-02-10 18:12:49 fclement Exp $*)

(* Numbering indices. *)

module Index = Uindex_index;;
module Hashtbl = Uindex_hashtbl;;

let iu_qual = Upervasives.qualify "Uindex_universe";;

let iu_invalid_arg = Upervasives.invalid_arg_qual iu_qual;;

type 'a element = 'a Hashtbl.key;; (* = 'a Uindex.t *)
type 'a index = 'a Hashtbl.value;; (* = 'a element Uindex.t *)

type 'a t =
  {
   tables : 'a Hashtbl.t;
   cardinal : Upervasives.length;
  }
;;

(* Construction functions. *)

let of_array a =
  let s1h = Hashtbl.make a in
  { tables = s1h; cardinal = Hashtbl.length s1h; }
;;

let of_list1 l1 = of_array (Uarray.of_list1 l1);;

let of_nonempty_range l r =
  if l >= r then iu_invalid_arg "of_nonempty_range" "empty range" else
  of_list1 (Ulist1.of_range Uindex.pred l r)
;;

(* Access functions. *)

let cardinal iu = iu.cardinal;;

let mem iu = Hashtbl.mem iu.tables;;

let index iu = Hashtbl.value_of_key iu.tables;;

let element iu = Hashtbl.key_of_value iu.tables;;

let elements iu =
  Uarray.init (cardinal iu) (fun i -> element iu (Index.of_int i))
;;

let equal iu1 iu2 =
  iu1 == iu2 ||
  Hashtbl.equal iu1.tables iu2.tables
;;

(* Iterators. *)

let iter f iu = Hashtbl.iter (fun x idx -> f idx x) iu.tables;;

let sub_iter f iu = Hashtbl.sub_iter (fun x idx -> f idx x) iu.tables;;

let fold_left f accu iu =
  Hashtbl.fold_left (fun accu x idx -> f accu idx x) accu iu.tables
;;
